//car.h - This file is meant to be left as is
// exposes public function calls into car.cpp

////////////////////////////////////////////////////////////////////////////////
//  car.cpp and car.h contain all core functions for webpage, WiFi, driving
//      The user does not need to change this code, except for settings at top

#ifndef car_h
#define car_h

#include <ESPAsyncWebServer.h>
#include <WString.h>
#include "DNSServer.h"
#include <Adafruit_NeoPixel.h>

#define BLINK_PIN 22
#define EXTRA_PIN 19
#define SERVO_PIN 17 // was pin 16 before

#define MOTOR_PIN_L1 18  // Motor PWM channels are 5, 6, 7, 8  
#define MOTOR_PIN_L2  5
#define MOTOR_PIN_R1 13
#define MOTOR_PIN_R2 15

#define NEOPIXEL_PIN 23
#define NEOPIXEL_NUMPIXELS 8

#define RGB_COMMON_PIN 0
#define RGB_R_PIN 2
#define RGB_G_PIN 4
#define RGB_B_PIN 16
#define RGB_R_CHANNEL 9  // PWM channels to use; motors use 5, 6, 7, 8
#define RGB_G_CHANNEL 10
#define RGB_B_CHANNEL 11
// For network settings, WiFi name see car.cpp


namespace car
{  
  void initCar();

  void IRAM_ATTR checkTimeout();
  void dnsTask( void * parameter );
  void task(void * pvParameters);
  void onWebsocket(AsyncWebSocket * server, AsyncWebSocketClient * client, AwsEventType type, void * arg, uint8_t *data, size_t len);

  int getTrottle();
  int getRotation();
  void writeMotorL(int motorTrottle);
  void writeMotorR(int motorTrottle);
  void writeMotors(int motorL, int motorR);
  void initMotor();
  int toMotorSpeed(int x);

  bool neopixel_is_available();
  
};

#endif
