// functions.cpp - This file is meant to be edited to add custom actions

////////////////////////////////////////////////////////////////////////////////
//  functions.cpp and functions.h allow the user to extend some control items  
//      The user is invited to add custom code to create custom actions
//
// custom elements are: servo, RGB, blink, extra; pins as defined in car.cpp
// core elements are: motors, NeoPixel, networking, webpage; see car.cpp

#include <ESP32Servo.h>  // older version <ESP32_Servo.h>, newer version same library <ESP32Servo.h>
#include <Adafruit_NeoPixel.h>  //ESP can use a nicer library, however this conforms to our earlier examples
#include "car.h"



namespace car{
  //define the motorLSpeed and motorRSpeed variables as integers (numbers)
  int motorLSpeed;
  int motorRSpeed;

  Servo myservo;
  
  Adafruit_NeoPixel pixels_functions(NEOPIXEL_NUMPIXELS, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);
  // note: we use a different pixels definition in car.cpp for core functions
  
  void neopixel_nightrider_update(int r, int g, int b);
  void rgb_update(int r, int g, int b);
  void rgb_rainbow_update();
  int r = 250, g = 202, b = 48;  // RGB code for RAL 1018, Junior IOT yellow is 250, 202, 48

  void setupFunction(){
    // This code is called once at robot startup
    // setupFunction is the place to config all user defined extra's
    
    myservo.attach(SERVO_PIN, 500, 2400);  
    
    pinMode(EXTRA_PIN, OUTPUT); //optional flame thrower
    digitalWrite(EXTRA_PIN, false); 
    
    pinMode(BLINK_PIN, OUTPUT); //Turn on built in led
    
    pinMode(RGB_COMMON_PIN, OUTPUT); // Optional RGB
    digitalWrite(RGB_COMMON_PIN, true); // set common pin to always on 
    // configure LED PWM functionalitites  
    ledcSetup(RGB_R_CHANNEL, 500, 8);  //(ledChannel, freq, resolution); 
    ledcSetup(RGB_G_CHANNEL, 500, 8);  //(ledChannel, freq, resolution); 
    ledcSetup(RGB_B_CHANNEL, 500, 8);  //(ledChannel, freq, resolution);  
    // attach the channel to the GPIO to be controlled
    ledcAttachPin(RGB_R_PIN, RGB_R_CHANNEL);  //(ledPin, ledChannel);
    ledcAttachPin(RGB_G_PIN, RGB_G_CHANNEL);  //(ledPin, ledChannel);
    ledcAttachPin(RGB_B_PIN, RGB_B_CHANNEL);  //(ledPin, ledChannel);
    rgb_update(250,202,48);  //Junior IOT yellow is 250, 202, 48

    // INITIALIZE NeoPixel strip object (REQUIRED)
    pixels_functions.begin(); 
  }

  void loopFunction(){
    // This code is called repeatedly after startup
    // loopFunction is the place to continuously execute all user defined extra's
    // do not use Delay() to avoud disturbing other code in this loop

    //Blink NO DELAY for builtin led
    if ((millis()/100)%10 == 1) {
      digitalWrite(BLINK_PIN, false); // on
    } else {   
      digitalWrite(BLINK_PIN, true);  // off  
    }

    // update the RGB led according to current timer() value
    rgb_rainbow_update();

    // update the NeoPixel display if not in use by core functions
    if (neopixel_is_available()) {  
      neopixel_nightrider_update(r, g, b);
    }

    // user can add more to this loop 

    // delay helps free up the core
    delay(20);
  }
  
  void rgb_update(int r, int g, int b) {
    // changing the LED brightness with PWM - Led has common plus so 255 color value displays as dark
    ledcWrite(RGB_R_CHANNEL, 255-r);
    ledcWrite(RGB_G_CHANNEL, 255-g);
    ledcWrite(RGB_B_CHANNEL, 255-b);
  }
    
  int rainbow_rgb [7][3] = {
    { 148,   0, 211 },   // Violet 148, 0, 211
    {  75,   0, 130 },   // Indigo 75, 0, 130
    {   0,   0, 255 },   // Blue 0, 0, 255
    {   0, 255,   0 },   // Green 0, 255, 0
    { 255, 255,   0 },   // Yellow 255, 255, 0
    { 255, 127,   0 },   // Orange 255, 127, 0
    { 255,   0,   0 } };   // Red 255, 0 , 0

  int prev_step_rainbow = -1;  
  void rgb_rainbow_update() {    
    // main loop runs very frequently --> use millis() to calculate status
    unsigned long this_moment = millis();  
    const int time_each_color = 1000; // milisec
    const int num_steps = 7 ;  // 7 is number of rainbow colors
    int curr_step = (this_moment/time_each_color) % num_steps;    
    if (curr_step != prev_step_rainbow) {  
      // we need to display a new step
      int this_color=curr_step;        
      // rgb_update() takes RGB values, from 0,0,0 up to 255,255,255
      rgb_update( rainbow_rgb[this_color][0], rainbow_rgb[this_color][1], rainbow_rgb[this_color][2]);
      
      prev_step_rainbow = curr_step;
    }  // if a change was needed
  }

  int prev_step_nightrider=-1;
  void neopixel_nightrider_update(int r, int g, int b) {
    // main loop runs very frequently --> use millis() to calculate status
    unsigned long this_moment = millis();
  
    const int num_steps = NEOPIXEL_NUMPIXELS * 2 - 2;  //this is 14 steps
    const int time_each_pixel = 2000/num_steps; // milisec
    int curr_step = (this_moment/time_each_pixel) % num_steps;
  
    if (curr_step != prev_step_nightrider) {
      pixels_functions.clear(); // Set all pixel colors to 'off'
  
      if (curr_step < NEOPIXEL_NUMPIXELS) {
        // step 0  1  2  3  4  5  6  7   --> go up from led 0 to 7
        int this_pixel=curr_step;
        
        // pixels.Color() takes RGB values, from 0,0,0 up to 255,255,255
        pixels_functions.setPixelColor(this_pixel, pixels_functions.Color(r, g, b));
      } else  {
        // step    8  9 10 11 12 13      --> go backwards from led 6 to 1
        int this_pixel=NEOPIXEL_NUMPIXELS * 2 - 2 - curr_step;
        
        // pixels.Color() takes RGB values, from 0,0,0 up to 255,255,255
        pixels_functions.setPixelColor(this_pixel, pixels_functions.Color(r, g, b));
      }
      
      pixels_functions.show();   // Send the updated pixel colors to the hardware.
      prev_step_nightrider = curr_step;
    }  // if a change was needed
  }

  void movement(int throttle, int rotation){
    // This code is called when the (virtual) joystick values change
    
    //calculate the motor speed
    motorLSpeed = throttle - rotation;
    motorRSpeed = throttle + rotation;
  
    //write the motor trottle values
    writeMotorL(motorLSpeed);
    writeMotorR(motorRSpeed);
  }

  void button1(){
    // This code runs when button 1 on the controller is pressed

    // adjust servo position
    myservo.write(0);  

    // adjust the NeoPixel Nightrider color
    r = 250, g = 202, b = 48;  // RGB code for RAL 1018, Junior IOT yellow is 250, 202, 48

    // apply some bonus output
    digitalWrite(EXTRA_PIN, true);      // flame thrower on
    
    Serial.println("button 1");
  }

  void button2(){
    // This code runs when button 2 on the controller is pressed
    myservo.write(60);
    r = 150, g = 0, b = 0; 
    Serial.println("button 2");
  }

  void button3(){
    // This code runs when button 3 on the controller is pressed
    myservo.write(120);
    r = 0, g = 150, b = 0; 
    Serial.println("button 3");
  }

  void button4(){
    // This code runs when button 4 on the controller is pressed
    myservo.write(180);
    r = 0, g = 0, b = 150; 
    digitalWrite(EXTRA_PIN, false);   // flame thrower off
    Serial.println("button 4");
  }
}
